from flask import Blueprint, render_template, request, flash, redirect, url_for, session, abort
from app import db
from app.models import Video, Category
from werkzeug.utils import secure_filename
from dotenv import load_dotenv
import uuid
import hashlib
import os

bp = Blueprint('main', __name__)

load_dotenv()

ADMIN_TOKEN = os.getenv("ADMIN_TOKEN")

UPLOAD_FOLDER = "app/static/uploads"
PREVIEW_FOLDER = "app/static/preview"

if not os.path.exists(UPLOAD_FOLDER):
    os.makedirs(UPLOAD_FOLDER)
if not os.path.exists(PREVIEW_FOLDER):
    os.makedirs(PREVIEW_FOLDER)

ALLOWED_EXTENSIONS = {'mp4', 'ogv', 'webm', 'png', 'jpg', 'jpeg'}

def allowed_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

def generate_hash(filename):
    hash_object = hashlib.md5(filename.encode())
    return hash_object.hexdigest()[:16]

def save_video_file(file, filename):
    unique_hash = generate_hash(filename)
    video_filename = f"{filename.rsplit('.', 1)[0]}_{unique_hash}-{uuid.uuid4()}.mp4"
    video_path = os.path.join(UPLOAD_FOLDER, video_filename)
    file.save(video_path)
    return video_filename, video_path

def save_preview_image(file, filename):
    unique_hash = generate_hash(filename)
    preview_filename = f"{filename.rsplit('.', 1)[0]}_{unique_hash}-{uuid.uuid4()}.png"
    preview_path = os.path.join(PREVIEW_FOLDER, preview_filename)
    
    try:
        file.save(preview_path)
    except Exception as e:
        print(f"Error saving preview image: {e}")
        return None
    
    return preview_filename, preview_path

@bp.route('/')
def index():
    videos = Video.query.all()
    categories = Category.query.all()
    return render_template('index.html', videos=videos, categories=categories)

@bp.route('/category/<int:category_id>')
def category(category_id):
    category = Category.query.get_or_404(category_id)
    videos = Video.query.filter_by(category_id=category.id).all()
    return render_template('index.html', videos=videos, categories=Category.query.all())

@bp.route('/delete_category/<int:category_id>', methods=['POST'])
def delete_category(category_id):
    if 'sessionIsActive' in session and session['sessionIsActive']:
        category = Category.query.get_or_404(category_id)
        
        for video in category.videos:
            video_path = os.path.join(UPLOAD_FOLDER, video.filename)
            if os.path.exists(video_path):
                os.remove(video_path)
            
            preview_path = os.path.join(PREVIEW_FOLDER, video.preview_filename)
            if os.path.exists(preview_path):
                os.remove(preview_path)

            db.session.delete(video)
        
        db.session.delete(category)
        db.session.commit()

        flash('Category and its videos deleted successfully!')
        return redirect(url_for('main.admin'))
    
    abort(403)

@bp.route('/<category_slug>/watch/<video_slug>')
def video_detail(category_slug, video_slug):
    video = (Video.query
             .join(Category)
             .filter(Category.slug == category_slug, Video.slug == video_slug)
             .first_or_404())
    return render_template('video_detail.html', video=video, categories = Category.query.all())

@bp.route(f'/admin', methods=['GET', 'POST'])
def admin():
    if 'sessionIsActive' in session and session['sessionIsActive']:
        if request.method == 'POST':
            if 'video' in request.files and 'preview' in request.files:
                video_file = request.files['video']
                preview_file = request.files['preview']

                if video_file and allowed_file(video_file.filename):
                    video_filename = secure_filename(video_file.filename)

                    if video_filename.strip() != "":
                        video_filename, video_path = save_video_file(video_file, video_filename)

                        if preview_file and allowed_file(preview_file.filename):
                            preview_filename, preview_path = save_preview_image(preview_file, video_filename)
                        else:
                            flash("Invalid preview image file type.")
                            return redirect(url_for('main.admin'))

                        category_id = request.form.get('category')
                        title = request.form.get('title')
                        description = request.form.get('description')

                        video_slug = Video(title=title).generate_slug

                        video = Video(
                            title=title,
                            filename=video_filename,
                            preview_filename=preview_filename,
                            category_id=category_id,
                            slug=video_slug,
                            description=description
                        )
                        db.session.add(video)
                        db.session.commit()

                        flash('Video uploaded successfully!')
                    else:
                        flash('Invalid video filename.')
                else:
                    flash('Invalid video file type. Please upload a valid video file.')

            elif 'category_name' in request.form:
                category_name = request.form.get('category_name')
                if category_name.strip() != "":
                    category = Category(name=category_name)
                    db.session.add(category)
                    db.session.commit()

            return redirect(url_for('main.admin'))

        categories = Category.query.all()
        return render_template('admin.html', categories=categories)
    
    else:
        token = request.args.get('token')
        if token == ADMIN_TOKEN:
            session['sessionIsActive'] = True
            return redirect(url_for('main.admin'))
        else:
            abort(403)

@bp.route('/logout', methods=['POST'])
def logout():
    session.pop('sessionIsActive', None)
    flash('You have been logged out successfully.', 'success')
    return redirect(url_for('main.index'))
